package com.sf.smutils.util;

//MIT Public Domain License
//
//Copyright (c) 2014 Mark O'Donohue [ mark.odonohue@gmail.com ]
//All rights reserved.
//
//Permission is hereby granted, free of charge, to any person obtaining a copy
//of this software and associated documentation files (the "Software"), to deal
//in the Software without restriction, including without limitation the rights
//to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//copies of the Software, and to permit persons to whom the Software is
//furnished to do so, subject to the following conditions:
//
//The above copyright notice and this permission notice shall be included in all
//copies or substantial portions of the Software.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
//SOFTWARE.
//


import java.util.*;
import java.io.*;


public class RuntimeProperties {
		
    Properties runParameters = new Properties();
	
    String propertyFileName = "";

    public void load() {
    	load(propertyFileName);
    }
    
    public RuntimeProperties clone() {
        RuntimeProperties cloneSet = new RuntimeProperties();
        
        for (String name : runParameters.stringPropertyNames()) {           
            String value = runParameters.getProperty(name);
            cloneSet.getProperties().put(name, value);
      }
        return cloneSet; 
    }
    
    
    public void dumpProperties() {
    	for (String name : runParameters.stringPropertyNames()) {    		
			  String value = runParameters.getProperty(name);
			  System.out.println(name + "=" + value);
		}
    }

    public void load(String propertyFileName) {
        File loadFile = new File(propertyFileName);
        if (!loadFile.exists()) {
            throw new RuntimeException("Missing required property file : " + propertyFileName);
        }
        
        try {
	       FileInputStream input = new FileInputStream(loadFile);
	       runParameters.load(input);
        }
        catch (Exception e) {
        	throw new RuntimeException(e);
        }
    }

    public Properties getProperties() { return runParameters; }
    
    public String getProperty(String key)  {
        String value = getProperty(key, null);
        if (value == null) {
            throw new RuntimeException("Missing required property " + key + " from " + propertyFileName);
        }
        return value;
    }
    
    public String getProperty(String key, String defaultValue) {
        String value = runParameters.getProperty(key);
        if (value == null) {
            value = defaultValue;
        }
        return value;
    }

    public List<String> getPropertyAsStringList(String key) {
        List<String> value = getPropertyAsStringList(key, null); 
        
        if (value == null) {
            throw new RuntimeException("Missing required property '" + key + "' from " + propertyFileName);
        }
        
        return value;
    }
    
    public List<String> getPropertyAsStringList(String key, String defaultValue) {
        String value = runParameters.getProperty(key, null);
        if (value == null) {
            value = defaultValue;
        }
        
        // If the default was null, then return null, not an empty array.
        if (value == null) {
            return null;
        }
        
        List<String> valueList = new ArrayList<String>();
        
        // The split will return 1 element if it the string is blank. so this handles that.
        if (value.trim().length() != 0) {
            valueList.addAll(Arrays.asList(value.split("\\s+")));
        }
        return valueList;
    }
    

    /**
     * This method will collect either :
     *     BaseName=line1 
     *  or
     *     BaseName_1=line1
     *     BaseName_2=line2
     *  and return them as an array of strings.
     */
    
    public ArrayList<String> getPropertiesList(String baseKey, String defaultValue) {
        
        ArrayList<String> list = new ArrayList<String> (); 
        String str = getProperty(baseKey, null);
        
        if (str != null) {
            list.add(str);
        }

        int idx = 0;
        
        while (true) {
            idx++;
            String incKey = baseKey + "_" + Integer.toString(idx);
            str = getProperty(incKey, null);

            if (str == null) {
                break;
            }
            list.add(str);

        }

        // If default value was null then return that.
        if (list.size() == 0) {
            if (defaultValue == null) {
                return null;
            }
            else {
                list.add(defaultValue);
            }
        }
        
        return list;
    }
    
    /**
     * As above just default to returning an empty list.
     */
    public ArrayList<String> getPropertiesList(String baseKey) {
    	ArrayList<String> result = getPropertiesList(baseKey, null);
    	if (result == null) {
    		result = new ArrayList<String>(); 
    	}
    	return result;
    }
    
   public double getPropertyAsDouble(String key)  {
       String valueString = getProperty(key);
       double value = -1.0;
       try {
           value = Double.parseDouble(valueString);
       }
       catch (NumberFormatException  e) {
            throw new RuntimeException("Unable to parse number from " + value + "for key " + key + " in " + propertyFileName);
       }
       return value;
    }

   public double getPropertyAsDouble(String key, double defaultValue) {
       double value = defaultValue;

       String valueString = getProperty(key, null);
       if (valueString != null) {
           try {
               value = Double.parseDouble(valueString);
           }
           catch (NumberFormatException  e) {}
       }
       return value;
    }

   public int getPropertyAsInt(String key) {
       int value = -1;
       try {
	       String valueString = getProperty(key).trim();
           value = Integer.parseInt(valueString);
       }
       catch (NumberFormatException  e) {
            throw new RuntimeException("Unable to parse number from " + value + " for key " + key + " in " + propertyFileName);
       }
       return value;
    }

   public int getPropertyAsInt(String key, int defaultValue) {
       int value = defaultValue;

       String valueString = getProperty(key, null);
       if (valueString != null) {
           try {
               value = Integer.parseInt(valueString);
           }
           catch (NumberFormatException  e) {}
       }
       return value;
    }

   public long getPropertyAsLong(String key) {
       long value = -1;
       try {
	       String valueString = getProperty(key);
           value = Long.parseLong(valueString);
       }
       catch (NumberFormatException  e) {
            throw new RuntimeException("Unable to parse number from " + value + "for key " + key + " in " + propertyFileName);
       }
       return value;
    }

   public long getPropertyAsLong(String key, long defaultValue) {
       long value = defaultValue;

       String valueString = getProperty(key, null);
       if (valueString != null) {
           try {
               value = Long.parseLong(valueString);
           }
           catch (NumberFormatException  e) {}
       }
       return value;
    }
   
   public boolean getPropertyAsBoolean(String key, boolean defaultValue) {
       boolean value = defaultValue;

       String valueString = getProperty(key, null);
       if (valueString != null) {
           if (valueString.toLowerCase().startsWith("t")) {
               value = true;
           }
           else if (valueString.toLowerCase().startsWith("f")) {
               value = false;
           }               
       }
       return value;
    }
   

   public void setProperties(Properties properties) { 
       runParameters = properties;
   }
   
   public void setProperty(String name, String value) {
       if (value == null) {
           getProperties().remove(name);
       }
       else {
           getProperties().setProperty(name, value);
       }
   }
   
   public void setProperty(String name, boolean value) { setProperty(name, Boolean.toString(value)); }
   public void setProperty(String name, int value) { setProperty(name, Integer.toString(value)); }
   public void setProperty(String name, long value) { setProperty(name, Long.toString(value)); }
   public void setProperty(String name, double value) { setProperty(name, Double.toString(value)); }
   
}

